# Copyright (c) 2025, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import logging
import time
from pathlib import Path

from nemo_curator.datasets import DocumentDataset
from nemo_curator.modifiers.pii_modifier import PiiModifier
from nemo_curator.modules.modify import Modify
from nemo_curator.utils.distributed_utils import get_client, read_data, write_to_disk
from nemo_curator.utils.file_utils import get_batched_files
from nemo_curator.utils.script_utils import ArgumentHelper


def main(args):
    """Main function that performs PII de-identification given a batch of files"""
    logging.basicConfig(
        format="%(asctime)s %(levelname)s:%(message)s",
        level=logging.DEBUG,
        datefmt="%Y-%m-%d %H:%M:%S",
    )

    logging.debug("Beginning PII job")
    start_time = time.time()
    Path(args.output_data_dir).mkdir(parents=True, exist_ok=True)

    supported_entities = (
        args.supported_entities.split(",") if args.supported_entities else None
    )

    modifier = PiiModifier(
        language=args.language,
        supported_entities=supported_entities,
        anonymize_action=args.anonymize_action,
        hash_type=args.hash_type,
        chars_to_mask=args.chars_to_mask,
        masking_char=args.masking_char,
        new_value=args.new_value,
        batch_size=args.batch_size,
        device=args.device,
    )

    for file_names in get_batched_files(
        args.input_data_dir, args.output_data_dir, args.input_file_type, args.n_workers
    ):
        logging.info("Reading input files....")
        source_data = read_data(
            file_names,
            file_type=args.input_file_type,
            backend="pandas",
            add_filename=True,
        )
        dataset = DocumentDataset(source_data)
        logging.debug(f"Dataset has {source_data.npartitions} partitions")

        modify = Modify(modifier, text_field=args.text_field)
        modified_dataset = modify(dataset)
        write_to_disk(
            modified_dataset.df,
            args.output_data_dir,
            write_to_filename=True,
            output_type=args.output_file_type,
        )

    end_time = time.time()
    logging.debug(
        "Total time taken for PII job: %0.3f seconds" % (end_time - start_time)
    )


def attach_args(
    parser=argparse.ArgumentParser(
        """
        Main driver script for applying PII redaction on documents. Inputs are in the input-data-dir directory.
        This script will then perform PII detection and de-identification on each document within the corpus.
        """,
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
):
    argumentHelper = ArgumentHelper(parser)

    argumentHelper.add_arg_batch_size(
        default=2000, help="The batch size for processing multiple texts together."
    )
    argumentHelper.add_arg_input_data_dir(help="Directory containing the input files.")
    argumentHelper.add_arg_input_file_type()
    argumentHelper.add_arg_language(help="Language of input documents.")
    argumentHelper.add_arg_output_data_dir(
        help="The output directory to where redacted documents will be written."
    )
    argumentHelper.add_arg_output_file_type()
    argumentHelper.add_distributed_args()

    parser.add_argument(
        "--anonymize-action",
        type=str,
        default="replace",
        help="Anonymization action. Choose from among: redact, hash, mask, and replace.",
    )
    parser.add_argument(
        "--chars-to-mask",
        type=int,
        default=100,
        help="The number of characters to mask. Only applicable if anonymize action is mask.",
    )
    parser.add_argument(
        "--hash-type",
        type=str,
        default=None,
        help="The hash type. Choose from among: sha256, sha512, or md5.",
    )
    parser.add_argument(
        "--masking-char",
        type=str,
        default="*",
        help="The masking character. Only applicable if anonymize action is mask.",
    )
    parser.add_argument(
        "--new-value",
        type=str,
        default=None,
        help="The new value to replace with. Only applicable if anonymize action is replace.",
    )
    parser.add_argument(
        "--supported-entities",
        type=str,
        default=None,
        help="Comma separated list of PII entity types. None implies all supported types.",
    )
    parser.add_argument(
        "--text-field",
        type=str,
        default="text",
        help="The input field within each JSONL or CSV object on which the PII redactor will "
        'operate. By default, the redactor will operate on the "text" '
        'field but other fields can be specified such as "url" or "id".',
    )

    return parser


def console_script():
    parser = attach_args()
    args = parser.parse_args()
    client = get_client(**ArgumentHelper.parse_client_args(args))
    if not args.n_workers:
        args.n_workers = len(client.scheduler_info()["workers"])
    main(args)


if __name__ == "__main__":
    console_script()



import os

import requests

from nemo_curator.download.doc_builder import DocumentDownloader


class TedTalksDownloader(DocumentDownloader):
    def __init__(self, download_dir: str):
        super().__init__()

        if not os.path.isdir(download_dir):
            os.makedirs(download_dir)

        self._download_dir = download_dir
        print("Download directory: ", self._download_dir)

    def download(self, url_src: str, url_tgt: str, force=False) -> str:
        src_filename = os.path.basename(url_src)
        src_out = os.path.join(self._download_dir, src_filename)
        tgt_filename = os.path.basename(url_tgt)
        tgt_out = os.path.join(self._download_dir, tgt_filename)

        sides = ["src", "tgt"]
        urls = [url_src, url_tgt]
        output_files = {"src": src_out, "tgt": tgt_out}
        for side, url, out_file_key in zip(sides, urls, output_files):
            if os.path.exists(output_files[out_file_key]) and force is False:
                print(
                    f"File '{output_files[out_file_key]}' already exists, skipping download."
                )
            else:
                print(f"Downloading TED Talks dataset from '{url}'...")
                response = requests.get(url)

                with open(output_files[out_file_key], "wb") as file:
                    file.write(response.content)

        return output_files



import logging
import os

from dask.distributed import Client, LocalCluster
from helper import process_data

logging.basicConfig(format="%(asctime)s: %(message)s", level=logging.INFO)

DATA_BASE = os.environ.get("DATA_BASE")
INPUT_BASE = os.path.join(DATA_BASE, "raw/dolma-v1_7-cc-parquet")
OUTPUT_BASE = os.path.join(DATA_BASE, "processed/dolma-v1_7-cc-parquet")
CPU_WORKERS = os.environ.get("CPU_WORKERS")


if __name__ == "__main__":
    logging.info("Starting Dask cluster")
    cluster = LocalCluster(n_workers=CPU_WORKERS, processes=True, memory_limit="48GB")
    client = Client(cluster)
    logging.info(client)

    logging.info(f"Processing Dolma-CC")
    process_data(input_folder=INPUT_BASE, output_folder=OUTPUT_BASE, prefix="dolma-cc")
    logging.info("Done!")

    client.cluster.close()
    client.shutdown()



# Copyright (c) 2024, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import json
import os
import pathlib
import warnings
from functools import partial, reduce
from typing import List, Optional, Union

import dask.bag as db
import dask.dataframe as dd
import numpy as np
import pandas as pd
from dask import delayed

from nemo_curator.utils.distributed_utils import (
    read_data,
    single_partition_write_with_filename,
)

NEMO_CURATOR_HOME = os.environ.get(
    "NEMO_CURATOR_HOME", os.path.join(os.path.expanduser("~"), ".nemo_curator")
)


def mkdir(d):
    pathlib.Path(d).mkdir(parents=True, exist_ok=True)


def expand_outdir_and_mkdir(outdir):
    outdir = os.path.abspath(os.path.expanduser(outdir))
    mkdir(outdir)
    return outdir


def filter_files_by_extension(
    files_list: List[str],
    keep_extensions: Union[str, List[str]],
) -> List[str]:
    """
    Given a list of files, filter it to only include files matching given extension(s).

    Args:
        files_list: List of files.
        keep_extensions: A string (e.g., "json") or a list of strings (e.g., ["json", "parquet"])
            representing which file types to keep from files_list.

    """
    filtered_files = []

    if isinstance(keep_extensions, str):
        keep_extensions = [keep_extensions]

    file_extensions = [s if s.startswith(".") else "." + s for s in keep_extensions]

    for file in files_list:
        if file.endswith(tuple(file_extensions)):
            filtered_files.append(file)

    if len(files_list) != len(filtered_files):
        warnings.warn(f"Skipped at least one file due to unmatched file extension(s).")

    return filtered_files


def get_all_files_paths_under(
    root: str,
    recurse_subdirectories: bool = True,
    followlinks: bool = False,
    keep_extensions: Optional[Union[str, List[str]]] = None,
) -> List[str]:
    """
    This function returns a list of all the files under a specified directory.
    Args:
        root: The path to the directory to read.
        recurse_subdirecties: Whether to recurse into subdirectories.
                              Please note that this can be slow for large
                              number of files.
        followlinks: Whether to follow symbolic links.
        keep_extensions: A string or list of strings representing a file type
                   or multiple file types to include in the output, e.g.,
                   "jsonl" or ["jsonl", "parquet"].
    """
    if recurse_subdirectories:
        file_ls = [
            os.path.join(r, f)
            for r, subdirs, files in os.walk(root, followlinks=followlinks)
            for f in files
        ]
    else:
        # Only include files, not directories
        file_ls = [entry.path for entry in os.scandir(root) if entry.is_file()]

    file_ls.sort()

    if keep_extensions is not None:
        file_ls = filter_files_by_extension(file_ls, keep_extensions)

    return file_ls


# Using this for restarting jobs
# can lead to problems when there is an error while
# writing a file we can use the offset counter approach
# in jaccard shuffle as a more robust way to restart jobs
def get_remaining_files(
    input_file_path: str,
    output_file_path: str,
    input_file_type: str,
    output_file_type: Optional[str] = None,
    num_files: int = -1,
):
    """
    This function returns a list of the files that still remain to be read.

    Args:
        input_file_path: The path of the input files.
        output_file_path: The path of the output files.
        input_file_type: The type of the input files.
        output_file_type: The type of the output files.
        num_files: The max number of files to be returned. If -1, all files are returned.
    Returns:
        A list of files that still remain to be read.

    """
    if input_file_type == "pickle":
        return [input_file_path]

    if not os.path.exists(output_file_path):
        expand_outdir_and_mkdir(output_file_path)
    completed_files = [
        os.path.basename(entry.path) for entry in os.scandir(output_file_path)
    ]
    completed_files = set(completed_files)

    input_files = [
        entry.path
        for entry in os.scandir(input_file_path)
        if os.path.basename(entry.path)
        not in _update_filetype(completed_files, output_file_type, input_file_type)
    ]
    # Guard against non extension files if present in the input directory
    input_files = [f for f in input_files if f.endswith(input_file_type)]
    input_files.sort()

    len_written_files = len(completed_files)
    if num_files > 0:
        left_to_sample = max(num_files - len_written_files, 0)
    else:
        left_to_sample = len(input_files)

    input_files = input_files[:left_to_sample]
    return input_files


def _update_filetype(file_set, old_file_type, new_file_type):
    if old_file_type is None or new_file_type is None:
        return file_set

    if not old_file_type.startswith("."):
        old_file_type = "." + old_file_type
    if not new_file_type.startswith("."):
        new_file_type = "." + new_file_type

    if old_file_type == new_file_type:
        return file_set

    updated_file_set = {
        (
            f"{os.path.splitext(file)[0]}{new_file_type}"
            if file.endswith(old_file_type)
            else file
        )
        for file in file_set
    }
    return updated_file_set


def get_batched_files(
    input_file_path: str,
    output_file_path: str,
    input_file_type: str,
    batch_size: int = 64,
):
    """
    This function returns a batch of files that still remain to be processed.

    Args:
        input_file_path: The path of the input files.
        output_file_path: The path of the output files.
        input_file_type: The type of the input files.
        batch_size: The number of files to be processed at once
    Returns:
        A batch of files that are not in the output directory.
    """
    remaining_files = get_remaining_files(
        input_file_path, output_file_path, input_file_type
    )
    for i in range(0, len(remaining_files), batch_size):
        yield remaining_files[i : i + batch_size]


def write_dataframe_by_meta(
    df: pd.DataFrame,
    output_dir: str,
    metadata_field: str,
    remove_metadata: bool = False,
    output_type: str = "jsonl",
    include_values: List[str] = None,
    exclude_values: List[str] = None,
    filename_col: str = "file_name",
):
    counts = df[metadata_field].value_counts().to_dict()

    # Apply include_values or value_exclesion_filter if provided
    if include_values is not None and include_values:
        counts = {k: v for k, v in counts.items() if k in include_values}
    elif exclude_values is not None and exclude_values:
        counts = {k: v for k, v in counts.items() if k not in exclude_values}

    for meta_value in counts:
        meta_output_dir = expand_outdir_and_mkdir(os.path.join(output_dir, meta_value))
        meta_slice = df[df[metadata_field] == meta_value]

        if remove_metadata:
            meta_slice = meta_slice.drop(columns=[metadata_field])
        single_partition_write_with_filename(
            meta_slice,
            meta_output_dir,
            output_type=output_type,
            filename_col=filename_col,
        )

    return counts


def merge_counts(first: dict, second: dict):
    for ngram, count in second.items():
        first[ngram] = first.get(ngram, 0) + count

    return first


def write_record(
    input_dir: str,
    file_name: str,
    line: str,
    field: str,
    output_dir: str,
    include_values: List[str] = None,
    exclude_values: List[str] = None,
):
    try:
        # Parse the JSON-encoded string 'line' into a Python dictionary
        line = json.loads(line)

        # Select category value
        category = line[field]

        if (exclude_values and category in exclude_values) or (
            include_values and category not in include_values
        ):
            return None

        # Obtain the relative path
        rel_path, file_name = os.path.split(
            os.path.relpath(file_name, start=os.path.abspath(input_dir))
        )

        output_dir = os.path.join(output_dir, category, rel_path)
        os.makedirs(output_dir, exist_ok=True)
        with open(os.path.join(output_dir, file_name), "a") as f:
            f.write(json.dumps(line) + "\n")

        return category
    except (KeyError, ValueError, json.JSONDecodeError):
        return None


def separate_by_metadata(
    input_data: Union[dd.DataFrame, str],
    output_dir: str,
    metadata_field: str,
    remove_metadata: bool = False,
    output_type: str = "jsonl",
    input_type: str = "jsonl",
    include_values: List[str] = None,
    exclude_values: List[str] = None,
    filename_col: str = "file_name",
) -> dict:
    """
    Saves the dataframe to subfolders named after a metadata

    Args:
        input_data: Either a DataFrame or a string representing the path to the input directory.
            If a DataFrame is provided, it must have a filename_col for the shard.
        output_dir: The base directory for which all metadata based subdirs will be created under
        metadata_field: The metadata field to split on
        remove_metadata: Whether to remove the metadata from the dataframe when saving it
        output_type: File type the dataset will be written to. Supported file formats include 'jsonl' (default),
            'pickle', or 'parquet'. (default: jsonl)
        include_values: A list of strings representing specific values to be selected or included.
            If provided, only the items matching these values should be kept.
        exclude_values: A list of strings representing specific values to be excluded or ignored.
            If provided, any items matching these values should be skipped.
        filename_col: The column name in the DataFrame that contains the filename. Default is "file_name".

    Returns:
        A delayed dictionary mapping each metadata to the count of entries with that metadata value.
    """

    if include_values is not None and exclude_values is not None:
        print("Error: 'include_values' and 'exclude_values' are mutually exclusive.")

        return

    # Create output_dir if needed
    if output_dir:
        output_dir = expand_outdir_and_mkdir(output_dir)

    if isinstance(input_data, str):
        print(f"Reading {input_type} files from {input_data}", flush=True)

        if input_type in ["json", "jsonl"] and output_type in ["json", "jsonl"]:
            # Read JSONL files with streaming (line-by-line), and include file path
            bag = db.read_text(
                os.path.join(input_data, "**", f"*.{input_type}"),
                include_path=True,
            )

            # Parse JSON lines and retain the file path
            bag = bag.map(
                lambda x: write_record(
                    input_dir=input_data,
                    file_name=x[1],
                    line=x[0],
                    field=metadata_field,
                    output_dir=output_dir,
                    include_values=include_values,
                    exclude_values=exclude_values,
                )
            )

            frequencies = dict(bag.frequencies().compute())
            frequencies.pop(None, None)  # Remove None when applying filters

            return delayed(reduce)(merge_counts, [frequencies])
        else:
            input_data = read_data(
                get_all_files_paths_under(input_data),
                file_type=input_type,
                backend="pandas",
                add_filename=filename_col,
            )
    delayed_counts = [
        delayed(write_dataframe_by_meta)(
            partition,
            output_dir,
            metadata_field,
            remove_metadata,
            output_type,
            include_values,
            exclude_values,
            filename_col,
        )
        for partition in input_data.to_delayed()
    ]

    return delayed(reduce)(merge_counts, delayed_counts)


def parse_str_of_num_bytes(s: str, return_str: bool = False) -> Union[str, int]:
    try:
        last_char = s[-1].lower()
        if last_char not in "kmg":
            raise ValueError("Invalid size: {}".format(s))
        power = "kmg".find(last_char) + 1
        size = float(s[:-1]) * 1024**power
    except ValueError:
        raise ValueError("Invalid size: {}".format(s))
    if return_str:
        return s
    else:
        return int(size)


def _save_jsonl(documents, output_path, start_index=0, max_index=10000, prefix=None):
    """
    Worker function to write out the data to jsonl files

    """

    def _encode_text(document):
        return document.strip().encode("utf-8")

    def _name(start_index, npad, prefix, i):
        tag = str(start_index + i).rjust(npad, "0")
        return f"{prefix}{tag}"

    # Create the naming function
    npad = int(np.log10(max_index) + 1)
    name = partial(_name, start_index, npad, prefix)

    output_glob_string = os.path.join(output_path, "*.jsonl")

    output_files = documents.map(_encode_text).to_textfiles(
        output_glob_string,
        name_function=name,
    )

    # Delete empty files generated due to empty partitions in the bag
    for output_file in output_files:
        try:
            if os.path.getsize(output_file) == 0:
                os.remove(output_file)
        except Exception as exception:
            print(
                f"An exception occurred when trying to delete {output_file}.\n{exception}",
                flush=True,
            )


def reshard_jsonl(
    input_dir: str,
    output_dir: str,
    output_file_size: str = "100M",
    start_index: int = 0,
    file_prefix: str = "",
):
    """
    Reshards a directory of jsonl files to have a new (approximate) file size for each shard

    Args:
        input_dir: The input directory containing jsonl files
        output_dir: The output directory where the resharded jsonl files will be written
        output_file_size: Approximate size of output files. Must specify with a string and
            with the unit K, M or G for kilo, mega or gigabytes
        start_index: Starting index for naming the output files. Note: The indices may not
            be continuous if the sharding process would output an empty file in its place
        file_prefix: Prefix to use to prepend to output file number
    """

    # Output file size in bytes
    blocksize = parse_str_of_num_bytes(output_file_size)

    input_files = list(get_all_files_paths_under(input_dir, keep_extensions="jsonl"))

    # Read in the dask bag
    b = db.read_text(input_files, blocksize=blocksize)

    # Prepare the output
    output_dir = expand_outdir_and_mkdir(output_dir)

    # Save to balanced files
    _save_jsonl(b, output_dir, start_index=start_index, prefix=file_prefix)


def remove_path_extension(path: str):
    p = pathlib.Path(path)
    return os.path.join(p.parent, p.stem)



# Copyright (c) 2024, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import os
import time

from nemo_curator import ConnectedComponents
from nemo_curator.utils.distributed_utils import get_client
from nemo_curator.utils.script_utils import ArgumentHelper


def main(args):
    """
    Takes a dataset consisting of document pairs
    and their corresponding Jaccard similarity scores to compute connected
    components of documents across pairs, to find similar documents
    after applying a given threshold. The result is a dataset
    consisting of all documents that are similar (above the threshold)
    and the component they belong to.
    """
    st = time.time()
    output_path = os.path.join(args.output_dir, "connected_components.parquet")
    args.enable_spilling = True
    client = get_client(**ArgumentHelper.parse_client_args(args))

    components_stage = ConnectedComponents(
        cache_dir=args.cache_dir,
        jaccard_pairs_path=args.jaccard_pairs_path,
        id_column=args.input_json_id_field,
        jaccard_threshold=args.jaccard_threshold,
        logger=args.log_dir,
        profile_dir=args.profile_path,
    )
    components_stage.cc_workflow(output_path=output_path)
    print(f"All done in {time.time()-st:.1f} seconds")
    print(f"Results written to {output_path}")


def attach_args():
    description = "Computes connected components."
    parser = argparse.ArgumentParser(
        description,
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )
    argumentHelper = ArgumentHelper(parser)

    argumentHelper.parse_gpu_dedup_args()

    argumentHelper.add_arg_output_dir()
    parser.add_argument(
        "--cache-dir",
        type=str,
        help="The cache directory to write intermediate results to.",
    )
    parser.add_argument(
        "--jaccard-pairs-path",
        type=str,
        help="The directory containing the Jaccard results.",
    )
    parser.add_argument(
        "--jaccard-threshold",
        type=float,
        default=0.8,
        help="Jaccard threshold below which we do not consider documents"
        " to be duplicates.",
    )

    return parser


def console_script():
    main(attach_args().parse_args())


if __name__ == "__main__":
    main(attach_args().parse_args())



# Copyright (c) 2024, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import gzip
import os
import re
import subprocess
import tarfile
import tempfile
from typing import Literal, Optional

from nemo_curator.datasets import DocumentDataset
from nemo_curator.download.doc_builder import (
    DocumentDownloader,
    DocumentExtractor,
    DocumentIterator,
    download_and_extract,
)
from nemo_curator.utils.download_utils import get_arxiv_urls
from nemo_curator.utils.file_utils import (
    expand_outdir_and_mkdir,
    get_all_files_paths_under,
)

# The iterator and extractor code are in large part taken
# from the Red-Pajama repo
# https://github.com/togethercomputer/RedPajama-Data/tree/main/data_prep/arxiv


class ArxivDownloader(DocumentDownloader):

    def __init__(self, download_dir, verbose=False):
        super().__init__()
        self._download_dir = download_dir
        self._verbose = False

    def download(self, tarfile):
        output_file = os.path.join(self._download_dir, tarfile)
        s3path = os.path.join("s3://arxiv/src", tarfile)
        if os.path.exists(output_file):
            print(f"tar file: {output_file} exists. Not downloading")
        else:
            print(f"Downloading {s3path} and writing to {output_file}")
            cmd = ["s5cmd", "--request-payer=requester", "cp", s3path, output_file]
            if self._verbose:
                stdout, stderr = None, None
            else:
                stdout, stderr = subprocess.DEVNULL, subprocess.DEVNULL
            p = subprocess.run(
                cmd,
                stdout=stdout,
                stderr=stderr,
            )
            if p.returncode != 0:
                print(f"Failed to download {s3path} to {output_file}")

        return output_file


class ArxivIterator(DocumentIterator):

    def __init__(self, log_frequency=1000):
        super().__init__()
        self._log_frequency = log_frequency
        self._counter = 0

    def iterate(self, file_path):
        self._counter = 0
        download_dir = os.path.split(file_path)[0]
        bname = os.path.split(file_path)[-1]
        with tempfile.TemporaryDirectory(dir=download_dir) as tmpdir:
            with tarfile.open(file_path) as tf:
                tf.extractall(members=tf.getmembers(), path=tmpdir)
                for i, item in enumerate(get_all_files_paths_under(tmpdir)):
                    if self._counter > 0 and self._counter % self._log_frequency == 0:
                        print(f"Extracted {self._counter} papers from {file_path}")
                    self._counter += 1

                    tex_files = self._tex_proj_loader(item)
                    arxiv_id = os.path.splitext(os.path.split(item)[-1])[0]

                    # get the arxiv id in the correct format
                    try:
                        clean_arxiv_id = self._format_arxiv_id(arxiv_id)
                    except Exception as e:
                        print(
                            f"[WARNING] failed to format arxiv id "
                            f"{arxiv_id}; exception={e}"
                        )
                        clean_arxiv_id = arxiv_id

                    if tex_files is None:
                        continue

                    yield {"id": clean_arxiv_id, "source_id": f"{bname}"}, tex_files

    def _tex_proj_loader(self, file_or_dir_path):
        r"""function to load the tex files from a tar file or a gzip file. The
        function will return a tuple containing a list of tex files and the
        timestamp of the project.

        @param file_or_dir_path: path to the tar file or the gzip file

        @return: tuple containing a list of tex files and the timestamp of the
            project
        """
        files_and_content = []

        try:
            # if it is a directory, open it as a tarfile
            with tarfile.open(file_or_dir_path) as sub_tf:
                for member in sub_tf.getmembers():
                    if member.name.endswith(".tex"):

                        file_content = sub_tf.extractfile(member).read()

                        try:
                            file_content = file_content.decode("utf-8")
                        except UnicodeDecodeError:
                            # self._logger.info(f"UnicodeDecodeError: {file_or_dir_path}")
                            return None

                        files_and_content.append(file_content)

        except tarfile.ReadError:
            # otherwise we try opening it as a gzip file
            try:
                with gzip.open(file_or_dir_path, "rb") as gz:
                    file_content = gz.read()
            except Exception:
                # all fails, we skip this file
                # self._logger.info(f"[ERROR] {e}: {file_or_dir_path}")
                return None

            try:
                file_content = file_content.decode("utf-8")
            except UnicodeDecodeError:
                # self._logger.info(f"UnicodeDecodeError: {file_or_dir_path}")
                return None

            files_and_content.append(file_content)

        except Exception as e:
            print(f"[ERROR] {e}: {file_or_dir_path}")
            return None

        return files_and_content

    def _format_arxiv_id(self, arxiv_id):
        r"""this function brings the raw arxiv-id into a format compliant with the
        specification from arxiv. This is used to create the url to the arxiv
        abstract page.

        - Format prior to March 2007:
            <archive>/YYMMNNN where N is a 3-digit number
        - Format after March 2007: <archive>/YYMM.NNNNN where N is a
          5 (or 6)-digit number

        References: https://info.arxiv.org/help/arxiv_identifier.html

        @param arxiv_id: raw arxiv id which can be in one of the
                         following formats:
                         - <archive><YY><MM><NNN>
                         - <YY><MM><NNNNN|NNNNNN>

        @return: formatted arxiv id
        """
        match = re.search(r"^([a-zA-Z-]*)([\d\.]+)$", arxiv_id)

        if match is None:
            raise ValueError(f"Invalid arxiv id: {arxiv_id}")

        if match.group(1) == "":
            return match.group(2)

        return f"{match.group(1)}/{match.group(2)}"


class ArxivExtractor(DocumentExtractor):

    def __init__(self):
        super().__init__()

    def extract(self, content):
        if len(content) == 0:
            return None

        # build dictionaries that contain the definitions of all macros in all tex
        # files. This is later used to expand all macros used in the text with
        # their definitions, so that consistency among different authors is
        # ensured.

        non_arg_macros = {}
        for file_content in content:
            non_arg_macros.update(self._build_non_arg_macros_dict(file_content))

        # TODO: macros that take arguments are not supported yet
        arg_macros = {}

        # join multiple latex files with a newline character
        try:
            cleaned_latex_file_str = "\n".join(
                self._clean_tex_file(
                    file_content=file_content,
                    arg_macros=arg_macros,
                    non_arg_macros=non_arg_macros,
                )
                for file_content in content
            )
        except Exception:
            return None

        # Don't return meta
        if cleaned_latex_file_str is not None:
            if len(cleaned_latex_file_str) > 0:
                return {"text": cleaned_latex_file_str}

    def _clean_tex_file(self, file_content, arg_macros, non_arg_macros):
        r"""function takes a tex file as input and returns a cleaned version. The
         cleaned version is a concatenation of the tex files with the
        following modifications:

        - remove all comments (i.e. all lines starting with %)
        - remove everything before the first section-like header
        - remove everything after the first occurrence of either \appendix or
            \bibliography
        - inline-expand definitions and macros

        @param file_content: the content of the tex file as a string.

        @return: cleaned tex file as a string
        """
        # find the first occurence of a \section-like header and replace everything
        # before it with an empty string. This matches the following pattern:
        #   \<section-type>[optional-args]{name}
        pattern = r"^(.*?)("
        pattern += r"\\\bchapter\b\*?(?:\[(.*?)\])?\{(.*?)\}|"
        pattern += r"\\\bpart\b\*?(?:\[(.*?)\])?\{(.*?)\}|"
        pattern += r"\\\bsection\b\*?(?:\[(.*?)\])?\{(.*?)\}|"
        pattern += r"\\\bsubsection\b\*?(?:\[(.*?)\])?\{(.*?)\}|"
        pattern += r"\\\bsubsubsection\b\*?(?:\[(.*?)\])?\{(.*?)\}|"
        pattern += r"\\\bparagraph\b\*?(?:\[(.*?)\])?\{(.*?)\}"
        pattern += r"\\\bsubparagraph\b\*?(?:\[(.*?)\])?\{(.*?)\}"
        pattern += r")"

        # if no section like header is found, then we return an empty string
        if not re.search(pattern, file_content, flags=re.DOTALL):
            return ""

        # replace everything with the second group of the match (i.e. everything
        # after and including the section header)
        file_content = re.sub(
            pattern=pattern,
            repl=r"\2",
            string=file_content,
            flags=re.DOTALL,  # make sure that the dot matches also newlines
        )

        # remove all line comments
        file_content = re.sub(
            pattern=r"(?m)^%.*\n?",
            repl=r"",
            string=file_content,
            flags=re.MULTILINE,
        )

        # remove all in comments within a line
        file_content = re.sub(
            # pattern matches a "%" that is not preceded by a backslash (=comment)
            pattern=r"[^\\]%.+$",
            repl=r"",
            string=file_content,
            flags=re.MULTILINE,
        )

        # find the first occurence of either \appendix or \bibliography and
        # replace everything after it with an empty string
        pattern = r"("
        pattern += r"\\appendix|"
        pattern += r"\\begin\{references\}|"
        pattern += r"\\begin\{REFERENCES\}|"
        pattern += r"\\begin\{thebibliography\}|"
        pattern += r"\\bibliography\{.*\}"
        pattern += r").*$"

        file_content = re.sub(
            pattern=pattern,
            repl=r"",
            string=file_content,
            flags=re.DOTALL,  # make sure that the dot matches also newlines
        )

        # inline-expand all non-arg macros
        for macro_name, macro_value in non_arg_macros.items():
            file_content = re.sub(
                # make pattern grouped to make sure that the macro is not part
                # of a longer alphanumeric word
                pattern=r"(" + macro_name + r")" + r"([^a-zA-Z0-9])",
                # replace the macro with its value and add back the character that
                # was matched after the macro
                repl=macro_value + r"\2",
                string=file_content,
            )

        # inline-expand all macros that use args
        # TODO: inline-expand macros with args
        for macro_name, macro_value in arg_macros.items():
            pass

        return file_content

    def _build_non_arg_macros_dict(self, file_content):
        r"""function takes the content of a tex file and returns a dictionary
        that contains the definitions of all macros that do not use arguments.
        The dictionary is of the form {macro_name: macro_value}.

        @param file_content: the content of the tex file as a string.

        @return: dict
        """
        # regex for extracting \newcommand macros without arguments
        non_arg_nc_reg = re.compile(
            # this regex matches the following:
            # \newcommand{\macro_name}{macro_value}
            # \newcommand*{\macro_name}{macro_value}
            # where macro_name is only allowed to contain letters and numbers;
            # macro_value can contain any character.
            pattern=r"\\\bnewcommand\b\*?\{(\\[a-zA-Z0-9]+?)\}\{(.*?)\}$",
            flags=re.MULTILINE,
        )

        # regex for extracting \def macros without arguments
        non_arg_def_reg = re.compile(
            # this regex matches the following:
            # \def\macro_name{macro_value}
            # where macro_name is only allowed to contain letters and numbers;
            # macro_value can contain any character.
            pattern=r"\\def\s*(\\[a-zA-Z0-9]+?)\s*\{(.*?)\}$",
            flags=re.MULTILINE,
        )

        # Extract all user-defined LaTeX macros from the preamble
        macros = {}
        for reg in [non_arg_nc_reg, non_arg_def_reg]:
            for match in reg.finditer(file_content):
                # convert the macro name and value to a raw string that can be
                # used in re.sub
                macro_name = match.group(1).encode("unicode-escape").decode("utf-8")
                macro_val = match.group(2).encode("unicode-escape").decode("utf-8")

                macros[macro_name] = macro_val

        return macros


def download_arxiv(
    output_path: str,
    output_type: Literal["jsonl", "parquet"] = "jsonl",
    raw_download_dir: Optional[str] = None,
    keep_raw_download: bool = False,
    force_download: bool = False,
    url_limit: Optional[int] = None,
    record_limit: Optional[int] = None,
) -> DocumentDataset:
    """
    Download Arxiv tar files and extract the contained LaTeX projects.

    This function obtains a list of Arxiv tar file URLs (via get_arxiv_urls), downloads the tar files,
    and then extracts the contained LaTeX source files. The resulting documents (after extraction) are
    assembled into a DocumentDataset.

    Args:
        output_path (str):
            The root directory where both the final extracted files and the raw download subdirectory will be stored.
            The extracted files (in the format specified by output_type) are eventually saved in this directory.
        output_type (Literal["jsonl", "parquet"], optional):
            The file format/extension used for saving the extracted documents (e.g., "jsonl" or "parquet").
            Default is "jsonl". This is not used for the output file, but is used to check if an extracted output already exists and read it if so.
        raw_download_dir (Optional[str], optional):
            The directory where the raw downloaded tar files will be kept. If None, a folder named "downloads"
            under output_path is used.
        keep_raw_download (bool, optional):
            If True, the raw tar files (before extraction) are not removed after processing. Default is False.
        force_download (bool, optional):
            If False, then if an output file already exists for a given URL, re-downloading and re-extraction will be skipped.
            Default is False.
        url_limit (Optional[int], optional):
            Limits the maximum number of Arxiv tar file URLs to download and process.
            If None, all available URLs (from get_arxiv_urls) are processed.
        record_limit (Optional[int], optional):
            Limits the maximum number of records to extract from each tar file.
            If None, all available records are extracted.
    Returns:
        DocumentDataset:
            A dataset object containing the extracted documents.
    """
    arxiv_urls = get_arxiv_urls()
    if url_limit:
        arxiv_urls = arxiv_urls[:url_limit]
    output_paths = list(
        map(lambda url: os.path.join(output_path, f"{url}.{output_type}"), arxiv_urls)
    )

    if not raw_download_dir:
        raw_download_dir = os.path.join(output_path, "downloads")
    expand_outdir_and_mkdir(raw_download_dir)
    downloader = ArxivDownloader(raw_download_dir)
    iterator = ArxivIterator()
    extractor = ArxivExtractor()

    output_format = {
        "text": str,
        "id": str,
        "source_id": str,
        "file_name": str,
    }
    dataset = download_and_extract(
        arxiv_urls,
        output_paths,
        downloader,
        iterator,
        extractor,
        output_format,
        output_type=output_type,
        keep_raw_download=keep_raw_download,
        force_download=force_download,
        filename_col="file_name",
        record_limit=record_limit,
    )

    return dataset



# Copyright (c) 2024, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
import warnings

from bs4 import BeautifulSoup, MarkupResemblesLocatorWarning

from nemo_curator.modifiers import DocumentModifier

# Ignore the specific BeautifulSoup warning
warnings.filterwarnings("ignore", category=MarkupResemblesLocatorWarning)


class CleanHTML(DocumentModifier):
    """
    A simple modifier that removes HTML tags from the document.
    """

    def modify_document(self, text: str) -> str:
        """
        Removes HTML tags from the document.

        Args:
            text (str): The text to be modified.

        Returns:
            str: The modified text.
        """
        text = BeautifulSoup(text, "lxml").get_text()
        # Remove extra whitespaces and newlines.
        return re.sub(r"\s+", " ", text).strip()



# Copyright (c) 2024, NVIDIA CORPORATION.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from .c4 import BoilerPlateStringModifier
from .doc_modifier import DocumentModifier
from .fasttext import FastTextLabelModifier
from .line_remover import LineRemover
from .markdown_remover import MarkdownRemover
from .newline_normalizer import NewlineNormalizer
from .pii_modifier import PiiModifier
from .quotation_remover import QuotationRemover
from .slicer import Slicer
from .unicode_reformatter import UnicodeReformatter
from .url_remover import UrlRemover

__all__ = [
    "DocumentModifier",
    "BoilerPlateStringModifier",
    "FastTextLabelModifier",
    "UnicodeReformatter",
    "QuotationRemover",
    "LineRemover",
    "MarkdownRemover",
    "PiiModifier",
    "NewlineNormalizer",
    "UrlRemover",
    "Slicer",
]
